import os, sys

#
# Same as exp0023, but using min dur of 2.5 secs
#
#

# Email address for notification
emailaddr = "wooters@icsi.berkeley.edu"

# Run-command parameters
rcparams = "-J 12"

# System parameters
expid  = "exp0023_001"           # base experiment id
ftype  = "mfcc19-10-30"          # feature type
nfeats = "19"                    # number of features per frame
frtime = "0.01"                  # duration in seconds
sc     = "16"                    # Number of starting clusters
mindur = "2.5"                   # Minimum duration of a cluster
ngauss = "05"                    # Number of initial gaussians per cluster

# A string describing the parameters of this experiment. This will be
# used as the name of a directory (under the 'topexpdir') in which all of the
# files for this experiment will be stored.
sysid  = expid+"_"+ftype+"_"+sc+"i_"+mindur+"s_"+ngauss+"g"

# Paths
basepath = "/u/wooters/projects/MeetingEval06/spkrseg"
clustercodepath = os.path.join(basepath,"exp0021code") # cluster source code path
clusterscriptpath = os.path.join(basepath,"scripts") # cluster scripts path
topexpdir = os.path.join(basepath,"expt")     # top-level experiment dir
expdir = os.path.join(topexpdir,sysid)  # mid-level experiment dir
                                        # This will contain scripts, code, etc.
outputdir = os.path.join(expdir,"output") # low-level experiment output dir
                                          # This will contain just the
                                          # diarization output.
bindir = os.path.join(expdir,"bin")      # for binary executables
scptdir = os.path.join(expdir,"scripts") # for scripts

# Location of feature files
featpath = os.path.join(basepath,"feats","mdm_dev06", ftype) + "/"
# Suffix of feature file names. This string gets added to the end
# of the meeting file basenames to create the basenames of the feature files.
featsuff = "_seg.feat"

# Location of speech/nonspeech files
spnspdir = os.path.join(basepath,"spnspnew","expt","1600_02_070") + "/"
# suffix of speech/nonspeech files
spnspsuff = "_seg.spch"

# Name of script for kicking-off the experiment
exprunfile = os.path.join(expdir,"run-expt.sh")

# Specify the location for run-file that will be generated. This is the file
# that is given as the '-f' argument to 'run-command'
RF_OUTFILE = os.path.join(scptdir,"runcmds")

# Original and "executable" locations of scripts/progs we want to run.
# These will be copied into the (low level) experiment directory.
clusterbase = "cluster"
origcluster = os.path.join(clustercodepath,clusterbase)
execcluster = os.path.join(bindir,clusterbase)

segexpbase  = "segexp.py"
origsegexp  = os.path.join(clusterscriptpath,segexpbase)
execsegexp  = os.path.join(scptdir,segexpbase)

seg2rttmbase = "seg2rttm_spkr.py"
origseg2rttm = os.path.join(clusterscriptpath,seg2rttmbase)
execseg2rttm = os.path.join(scptdir,seg2rttmbase)

# The base names of the meetings to process along with their initial
# offsets. The offset indicates the amount of time that is missing
# from the beginning of the feature files.
fileinfo = {
    "ICSI_20000807-1000":"1147.35",
    "ICSI_20010208-1430":"97.44",
    "LDC_20011116-1400":"192.00",
    "LDC_20011116-1500":"3346.00",
    "NIST_20030623-1409":"2.00",
    "NIST_20030925-1517":"133.93",
    "AMI_20041210-1052":"46.853",
    "AMI_20050204-1206":"1270.031",
    "CMU_20050228-1615":"4.500",
    "CMU_20050301-1415":"6.621",
    "VT_20050304-1300":"549.004",
    "VT_20050318-1430":"787.630"
}

# Commands to be run before the individual command lines are created.
PRE_CMDS = [
    "mkdir -p "+bindir,
    "mkdir -p "+scptdir,
    "mkdir -p "+outputdir,    
    "cp " + origcluster + " " + execcluster,
    "cp " + origsegexp  + " " + execsegexp,
    "cp " + origseg2rttm  + " " + execseg2rttm,
    "cp THISFILE " + expdir,    # save a copy of THIS file
    "cd "+clustercodepath+"; tar czvf "+expdir+"/code.tgz "+"./*.{h,cc,cpp} ./Makefile"
    ]

# Commands to be run after the individual command lines are created.
POST_CMDS = []

# Global definitions dictionary
# These apply to all of the files
CLUSTER_GLOBALS = {
"NSTARTCLUS"  : sc,
"MINDUR"      : mindur,
"FRAMETIME"   : frtime,
"NFEATURES"   : nfeats,
"NGAUSSIANS"  : ngauss,
"EXPDIR"      : outputdir,
"SEGPROG"     : execcluster,
"CLUSTERSCPT" : execsegexp,
"SEG2RTTMSCPT" : execseg2rttm
}

# Per-file definitions
CLUSTER_RUN_FILES = [{"FILEBASENAME":x, \
                      "FEATUREFILE":featpath+x+featsuff, \
                      "OFFSET":fileinfo[x], \
                      "SPNSPFILE":spnspdir+x+spnspsuff} \
                     for x in fileinfo.keys()]

# Base Command Line. The global and per-file definitions will be substituted
# into this string to form each individual command line.
CMDLINE = "CLUSTERSCPT -n NSTARTCLUS -d MINDUR -f FRAMETIME " + \
          "-a NFEATURES -i FEATUREFILE " + \
          "-g NGAUSSIANS -o OFFSET -w EXPDIR -S SEGPROG " + \
          "-c SPNSPFILE " + \
          "-B FILEBASENAME -R SEG2RTTMSCPT"

##################################################
# Scoring scripts and args

# Specify the location for the score command file that will be generated.
# This is the file that contains the scoring command and options.
scorecmdfile = os.path.join(scptdir,"doscore")

# Specify the location of the score directory.
scoredir = os.path.join(expdir,"score")

# Specify the location of the ref RTTMs
refdir = os.path.join(basepath,"rttm","mdm_dev06_srifa")

# Specify the location of the NIST scoring UEM file
uemfile = os.path.join(basepath,"uem","dev06.uem")

mdevalwrap = "domdeval.py"
origmdevalwrap = os.path.join(clusterscriptpath,mdevalwrap)
execmdevalwrap = os.path.join(scptdir,mdevalwrap)
mdevalargs = "-afc -c 0.25"
mdeval = "md-eval-v19a.pl"
origmdeval = os.path.join(clusterscriptpath,mdeval)
execmdeval = os.path.join(scptdir,mdeval)
spkrsegerror = "spkrsegError"
origspkrsegerror = os.path.join(clusterscriptpath,spkrsegerror)
execspkrsegerror = os.path.join(scptdir,spkrsegerror)
resultsfile = os.path.join(expdir,"results.out")

#  This will create the command-line for running the scoring and put it
#  into scorecmdfile
SCORE_CMDS = [
    "rm -rf "+scorecmdfile,
    "mkdir -p "+scoredir,
    "cp " + origmdeval  + " " + execmdeval,
    "cp " + origmdevalwrap  + " " + execmdevalwrap,
    "cp " + origspkrsegerror  + " " + execspkrsegerror,    
    "echo "+execmdevalwrap+" -p "+execmdeval+" -a \\\" "+mdevalargs+" \\\""+ 
    " -e "+outputdir+
    " -s "+scoredir+
    " -r "+refdir+
    " -u "+uemfile+
    " >> "+scorecmdfile,
    "echo \""+execspkrsegerror+" "+scoredir+"/mdeval.out | tee "+resultsfile+
    "\" >> " + scorecmdfile,
    "echo \"cat " + resultsfile + " | mail -s "+sysid+" "+emailaddr+
    " \" >> " + scorecmdfile
    ]
