//
// $Id: HTKfile.h,v 1.1.1.1 2005/05/06 01:15:31 xanguera Exp $
//
#ifndef __HTKFILE__
#define __HTKFILE__

#include <map>
#include <string>
#include <vector>

extern "C" {
#include "HShell.h"
#include "HMem.h"
#include "HAudio.h"
#include "HWave.h"
#include "HParm.h"
};

class windowBounds;

class HTKfile {

 public:
  HTKfile(){
    initDone = FALSE; openDone = FALSE;
    chunkinfo = 0;
    nSpOnly=0;
    spOnlyFlag=FALSE;
  }
  ~HTKfile() {DeleteHeap(&x);delete [] chunkinfo;}

  // This MUST be called before any other member function
  void init();

  // Open the HTK file pointed to by the given file name
  int open(const char *);

  // Read in the chunking info file
  void addChunkInfo(char*);
  unsigned char getChunkInfo(const int) const;

  // Read in the speech segments file. Must be called after open().
  void readUEM(const std::string);

  // Return the frame index in the original file that corresponds
  // to the given frame. 
  int mapFrame(const int) const;

  // Print header info
  void printinfo() const;

  // Return the observation at index i.
  // Caller must make sure the vector has enough space!
  void readObs(int, float*) const;

  // Return the number of components (features) per observation (frame)
  int get_nComps() const {return info.srcVecSize;}

  // Return the total number of observations (frames)
  int get_nObs() const;
  int get_nObsOrig() const;

  // Return the number of bytes per observation (frame)
  int get_sampleBytes() const {return info.srcVecSize*sizeof(float);}

  // Return the frame rate (in msecs) of the observations
  float get_samplePeriod() const {return info.srcSampRate/10000.0;}

  // Return the sample kind (e.g. LPCEPSTRA_K)
  // Caller must make sure buf has enough space.
  char* get_sampleKind(char* buf) const {return ParmKind2Str(info.srcPK,buf);}

  // Return the file format of this file
  const char* get_format() 
    const {return const_cast<const char*>(Format2Str(info.srcFF));}

  // Run windowed CMS on the features using a window length of 'winlen'
  void run_CMS(unsigned int winlen);

  // Run windowed CMN on the features using a window length of 'winlen'
  void run_CMN(unsigned int winlen);

 private:
  ParmBuf pbuf;
  BufferInfo info;
  MemHeap x;
  // this is mutable so that it can be modified from within const member
  // functions
  mutable Observation o;
  Boolean initDone;		// has init() been called?
  Boolean openDone;		// has open() been called?
  unsigned char *chunkinfo;

  // Checks to see if this machine uses vax byte ordering (opposite of
  // the Suns).
  Boolean IsVAXOrder(void);

  // A map that shows which frames in the speech-only data map to which
  // frames in the original file.
  std::map<int,int> frameMap;

  // Number of speech-only frames
  int nSpOnly;

  // Vector to hold vectors of speech-only frames. This should probably
  // be done with pointers to avoid using the extra memory. I'll have
  // to look at the HTK libraries to figure out how to do this....
  std::vector<std::vector<float> > spOnly;

  // Flag we use to decide if we are reading from the spOnly vector
  // or from the original file.
  Boolean spOnlyFlag;

  // Method to compute the window boundaries given the specified
  // window length. The boundaries are saved into the supplied 
  // vector 'bnds'.
  void getWinBounds(std::vector<windowBounds>& bnds, unsigned int winlen);

  // Compute the mean and variance of each feature over all feature 
  // vectors contained in the window whose bounds are given in 'wb'
  void getMeanVar(windowBounds wb, std::vector<double>& m, 
		  std::vector<double>& v);
  void getMean(windowBounds wb, std::vector<double>& m);
  void getGlobalVar(std::vector<double>& v);
  void fillSPOnly(std::vector<bool>& spflags);
};

#endif //__HTKFILE__
