//
// $Id: hmm.h,v 1.1.1.1 2005/05/06 01:15:31 xanguera Exp $
//
#ifndef __HMM__
#define __HMM__

#include "global.h"
#include "gmm.h"
#include "HTKfile.h"
#include <string.h>

class chunk {
public:
  int from;
  int to;
  int cluster;
  chunk *next;
  chunk *prev;
  ~chunk()
    {
      if(next!= NULL)
	delete next;
    }
};

class HMM {
public:
  int nClass;
  int DIM;			// dimensionality of feature vector
  int nGauss;			// initial number of gaussians per cluster
  int MIN_DURATION;
  int ALWAYS_POSSIBLE; // test for merge no matter what NP says
  float *lkld;
  GMM *gmm;
  int **NP;
  float **compared; // to be used when comparing neighbours to indicate wether they have been compared or not
  int *U; /* a pointer to indicate if a class is usable or not*/
  HMM(int K, int M, int DIM, int MD);
  ~HMM();
  HMM();

  float segment(const HTKfile& htkf, chunk **pFirst);
  void Calculate_Lkld(float *data);
  void Train_HMM(const HTKfile& htkf, chunk *first);
  void Train_HMM_Individual(const HTKfile& htkf, chunk *first, int cl);
  void Initialize(const HTKfile& htkf, chunk **first);
  void Initialize(const HTKfile& htkf, chunk **first, const char* const fname);
  void InitializeXBIC(const HTKfile& htkf, chunk **sent, int nClus);
  void Get_Data(const HTKfile& htkf, float ***pData, chunk *first, int cd1, int cd2, int *count);
  void Get_Data_noalloc(const HTKfile& htkf, float *pData, chunk *first, int cd1, int cd2);
  int Get_Nframes(const HTKfile& htkf, chunk *first, int cd1, int cd2);
  int Find_Best(int *mcd1, int *mcd2, chunk *first, const HTKfile& htkf);
  int Find_Best_Adjacent(int *mcd1, int *mcd2, chunk *first, const HTKfile& htkf);
  int Find_Best2(int *mcd1, int *mcd2, chunk *first, const HTKfile& htkf);
  int Find_N_Best(int *mcd1, int *mcd2, int *N, chunk *first, char *fname);
  void Merge(int mcd1, int mcd2, chunk *first, const HTKfile& htkf);
  void Merge_Segmentation(int mcd1, int mcd2, chunk *first, const HTKfile& htkf);
  void clearNP(int cd);
  void printNP();
  void Write(char *fname, chunk *first);
  void Write(const char *fname, const chunk * const first, 
	     const HTKfile& htkf);
  int nU(void);
  int if_possible(int cd1, int cd2);
  void PrintHMM(FILE *fout);
  void ReadHMM(FILE *fin, const int nC, const int d);
};

#endif
