#!/usr/bin/env python

import sys
import os
import string
import user
import getopt

#mdeval = "/u/drspeech/data/RT04/md-eval-v16-040625/src/md-eval-v16.pl"
mdeval = "/u/wooters/projects/autoseg2/eval_f04/scripts/md-eval-v19a.pl"
rttmsmooth = "/u/wooters/projects/MeetingEval06/spkrseg/scripts/rttmSmooth-v2.pl"
mdevalargs = " -c 0.25 -1 -a f -m "

exptdir = refdir = scoredir = uemfile = basekeep = None
filtnames = []
gapsmooth = 0.0

def usage():
    print "\nUsage:",sys.argv[0],"<options>",
    print """
Where <options> are:

    -e|--exptdir <expt_dir>            -the experiment you want to score
                                         (e.g. lpcc12-10-30_40i_05g_02s)
    -r|--refdir <reference_dir>        -the dir containing the ref rttm files
    -s|--scoredir <score_dir>          -the dir to hold the scored output
    -u|--uemfile <uem file>            -the NIST scoring uem file
    -g|--gapsmooth <minGapTime>        -remove within-speaker gaps if the gap
                                         duration is less than minGapTime
    -b|--basenames <list>              -list of basenames to score. [all]
    -p|--prog <script>                 -path to md-eval prog [%s]
    -a|--args <md-eval arg string>     -other md-eval args [\"%s\"]
"""%(mdeval,mdevalargs)
    
    sys.exit(1)
    
def parsecmdline():
    global exptdir, refdir, scoredir, uemfile, basekeep, filtnames, mdevalargs
    global gapsmooth
    
    cmdoptions = ["exptdir=","refdir=","scoredir=","uemfile=","basenames=",
                  "prog=","args=","gapsmooth="]
    if len(sys.argv) < 7:
        usage()
    try:
        opts,args = getopt.getopt(sys.argv[1:],"g:e:r:s:u:b:p:a:",cmdoptions)
    except getopt.GetoptError:
        print "Bad command line option"
        usage()

    for o,a in opts:
        if o in ("-g", "--gapsmooth"):
            gapsmooth = float(a)
        if o in ("-e", "--exptdir"):
            exptdir = a
        if o in ("-p", "--prog"):
            mdeval = a
        if o in ("-a", "--args"):
            mdevalargs = a
            print "Setting mdevalargs to [%s]"%mdevalargs
        if o in ("-r", "--refdir"):
            refdir = a
        if o in ("-s", "--scoredir"):
            scoredir = a
        if o in ("-u", "--uemfile"):
            uemfile = a
        if o in ("-b", "--basenames"):
            basekeep = a

    if gapsmooth < 0.0:
        sys.exit("Illegal value for --gapsmooth option (%0.4f)"%gapsmooth)
        
    if not os.path.exists(mdeval):
        print "Can't find md-eval prog: ",mdeval
        sys.exit(1)
    if exptdir == None:
        print "You must specify the experiment dir"
        usage()
    if not os.path.isdir(exptdir):
        print "Can't find experiment dir: ",exptdir
        sys.exit(1)
        
    if refdir == None:
        print "You must specify the reference dir"
        usage()
    if not os.path.isdir(refdir):
        print "Can't find reference dir: ",refdir        
        sys.exit(1)

    if scoredir == None:
        print "You must specify the score dir"
        usage()
    # make sure the scoredir exists
    if not os.path.isdir(scoredir):
        os.makedirs(scoredir)

    if uemfile == None:
        print "You must specify the UEM file"
        usage()
    if not os.path.isfile(uemfile):
        print "Can't find UEM file: ",uemfile
        sys.exit(1)

    if basekeep != None:
        if not os.path.isfile(basekeep):
            print "Can't find list of basenames: ",basekeep
            sys.exit(1);
        fh = open(basekeep)
        flines = fh.readlines()
        fh.close()
        flines = map(string.strip,flines)
        for f in flines:
            filtnames.append(f)
    
####################
if __name__ == '__main__':
    
    parsecmdline()

    # get a list file base names from the expt dir.
    # we are assuming that the experiment dir contains a list of subdirs
    # whose names are the names of the files we want to score
    basenames = os.listdir(exptdir)
    # only keep the directories, ignore any files
    basenames = [x for x in basenames if os.path.isdir(exptdir+"/"+x)]
    # only keep the basenames that are requested
    if len(filtnames) > 0:
        basenames = [x for x in basenames if x in filtnames]
        
    rttmfiles = [exptdir+"/"+x+"/"+x+".rttm" for x in basenames]
    refrttmfiles = [refdir+"/"+x+".rttm" for x in basenames]

    #
    # see if we need to smooth gaps, and if so, created the smoothed
    # rttm files
    #
    if gapsmooth > 0.0:
        smdir = scoredir+"/smoothed"
        if not os.path.isdir(smdir):
            os.makedirs(smdir)
        smrttmfiles = [smdir+"/"+x+".rttm" for x in basenames]
        for rttmpair in zip(rttmfiles,smrttmfiles):
            inrttm = rttmpair[0]
            outrttm = rttmpair[1]
            cmd = "grep SPEAKER %s | sort -k 8 -k 4n | %s -s %0.5f > %s"%(inrttm,
                                                                       rttmsmooth,
                                                                       gapsmooth,
                                                                       outrttm)
            rv = os.system(cmd)
            if rv != 0:
                print "Error creating smoothed rttm file:",outrttm
                sys.exit(1)

        # replace the original rttms with the smoothed rttms
        rttmfiles = smrttmfiles
        
    #
    # create a combined rttm file from the individual rttm files
    #
    print "Creating combined rttm file from individual rttm files...",
    sys.stdout.flush()
    combrttm = scoredir + "/all.rttm"
    if os.path.isfile(combrttm):
        os.remove(combrttm)
    rv = os.system("touch "+combrttm)
    if rv != 0:
        print "Error creating combined rttm file:",combrttm
        sys.exit(1)

    for i in rttmfiles:
        rv = os.system("cat "+i+" >> "+combrttm)
        if rv != 0:
            print "Error creating combined rttm file:",combrttm
            sys.exit(1)

    print "done."
    sys.stdout.flush()
        
    #
    # need to create a combined reference rttm file
    #
    print "Creating combined ref rttm file from individual ref rttm files...",
    sys.stdout.flush()
    combrefrttm = scoredir + "/all_ref.rttm"
    if os.path.isfile(combrefrttm):
        os.remove(combrefrttm)
    rv = os.system("touch "+combrefrttm)
    if rv != 0:
        print "Error creating combined ref rttm file:",combrefrttm
        sys.exit(1)
    for i in refrttmfiles:
        rv = os.system("cat "+i+" >> "+combrefrttm)
        if rv != 0:
            print "Error creating combined ref rttm file:",combrefrttm
            sys.exit(1)
    print "done."
    sys.stdout.flush()    

    #
    # now run the scoring
    #
    scoreoutfile = scoredir + "/mdeval.out"
    score_cmd = mdeval + mdevalargs + " -r " + combrefrttm + " -s " + \
                combrttm + " -u " + uemfile + " > " + scoreoutfile
    print "Running scoring command:\n"+score_cmd
    sys.stdout.flush()
    rv = os.system(score_cmd)
    if rv != 0:
        print "Error running scoring."
        sys.exit(1)

    sys.exit(0)
    
