#!/usr/bin/env python

import sys
import os
import string
import user
import getopt

def usage():
    print "\nUsage:", sys.argv[0], "<options>", 
    print """
Where <options> are:
    -s|--segfile <segfile>        -the seg.out file from cluster
    -b|--basename <basename>      -the basename of the file (e.g.
                                    20010228_2100_2200_MNB_NBW)
    -o|--offset <nsecs>           -start time offset in seconds [0.0]
    -f|--frpersec <nfr>           -number of frames per second [100.0]
"""
    sys.exit( 2 )

####################
if __name__ == '__main__':

    cmdoptions = ["segfile=", "basename=", "offset=", "frpersec="]
    if len( sys.argv ) < 4:
        usage()
    try:
        opts, args = getopt.getopt( sys.argv[1:], "s:b:o:f:", cmdoptions )
    except getopt.GetoptError:
        print "Bad command line option"
        usage()

    offset   = 0.0
    frpersec = 100.0
    basename = segfile = None

    for o, a in opts:
        if o in ( "-s", "--segfile" ):
            segfile = a
        if o in ( "-b", "--basename" ):
            basename = a
        if o in ( "-o", "--offset" ):
            offset = float( a )
        if o in ( "-f", "--frpersec" ):
            frpersec = float( a )

    if segfile == None:
        print "You must specify an input seg file"
        usage()
    if basename == None:
        print "You must specify a basename"
        usage()

    segfh = open( segfile )
    seglines = segfh.readlines()
    segfh.close()

    # first run through the seg file and make a list of all unique
    # "ids"
    speakers = {}
    for line in seglines:
        parts = line.split()
        spkr = parts[5]
        if spkr != "<nonspeech>":
            speakers[parts[5]] = 1

    # Now run through the list of speakers and print the SPKR-INFO 
    # lines
    spkrnames = speakers.keys()
    spkrnames.sort()
    for spkr in spkrnames:
        print "SPKR-INFO %s 1 <NA> <NA> <NA> unknown clus_%s <NA>"%( basename, 
                                                                spkr )
                                                                
    # run through the segfile and save all of the lines
    # in rttm format
    for line in seglines:
        parts = line.split()
        stfr = int( parts[1] )
        enfr = int( parts[3][:-1] )
        sp = parts[5]
        if sp != "<nonspeech>":
            sp = "clus_"+sp
            st = float( stfr )/frpersec + offset
            dur  = ( enfr - stfr + 1 )/frpersec
            #if (dur >= 0.5): #I don't want that for overlap meetings XAM
            print "SPEAKER %s 1 %0.3f %0.3f <NA> <NA> %s <NA>"%( basename, 
                                                                st, dur, sp )

        
    
